package com.ekexiu.project.bridge.servlet;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Type;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.jfw.util.ConstData;
import org.jfw.util.DateUtil;
import org.jfw.util.context.JfwAppContext;
import org.jfw.util.io.IoUtil;
import org.jfw.util.json.JsonService;
import org.jfw.util.reflect.TypeReference;

import com.ekexiu.project.bridge.collect.CollectDao;
import com.ekexiu.project.bridge.collect.CollectService;
import com.ekexiu.project.bridge.collect.WaveDataItem;

public class WaveDataServlet extends HttpServlet {
	private static final long serialVersionUID = 5218822832173871831L;

	private static final Map<String, List<WaveDataItem>> DATA_CACHE = new HashMap<String, List<WaveDataItem>>();

	@SuppressWarnings("unused")
	private static final LinkedList<List<WaveDataItem>> EMPTY_LIST = new LinkedList<List<WaveDataItem>>();

	public static Type DATA_TYPE = new TypeReference<Map<String, Object>>() {
	}.getType();

	private DataSource dataSource;
	private CollectDao collectDao;

	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		response.sendError(400);
	}

	@SuppressWarnings("unchecked")
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		InputStream in = request.getInputStream();
		final List<WaveDataItem> items = new LinkedList<WaveDataItem>();
		long currTime = System.currentTimeMillis();
		String ctime = DateUtil.formatDateTime(currTime);
		// String otime = DateUtil.formatDateTime(currTime -(5* 60*1000));
		String serverId = null;
		try {
			byte[] buf = new byte[8192];
			IoUtil.copy(in, baos, buf);
			Map<String, Object> map = JsonService.fromJson(new String(baos.toByteArray(), ConstData.UTF8), DATA_TYPE);
			Object tmpObj = map.get("node_id");
			if (null != tmpObj && (tmpObj instanceof String)) {
				serverId = (String) tmpObj;
				tmpObj = map.get("data");
				if (null != tmpObj && (tmpObj instanceof List)) {
					List<Object> ds = (List<Object>) tmpObj;
					for (Object wo : ds) {
						WaveDataItem item = new WaveDataItem();
						item.setCtime(ctime);
						item.setSeq(serverId);
						if (wo instanceof Map) {
							Map<Object, Object> im = (Map<Object, Object>) wo;
							tmpObj = im.get("box_id");
							if (tmpObj == null || (!(tmpObj instanceof Double))) {
								items.clear();
								break;
							}
							item.setDid(((Double) tmpObj).intValue());
							tmpObj = im.get("box_id");
							if (tmpObj == null || (!(tmpObj instanceof Double))) {
								items.clear();
								break;
							}
							item.setCid(((Double) tmpObj).intValue());
							tmpObj = im.get("value");
							if (null == tmpObj && (!(tmpObj instanceof List))) {
								items.clear();
								break;
							}
							List<Object> vs = (List<Object>) tmpObj;
							if (vs.size() != 2) {
								items.clear();
								break;
							}
							tmpObj = vs.get(0);
							if (tmpObj == null || (!(tmpObj instanceof List)) || (2 != ((List<?>) tmpObj).size())) {
								items.clear();
								break;
							}
							List<Object> vi = (List<Object>) tmpObj;
							tmpObj = vi.get(0);
							if (tmpObj == null || (!(tmpObj instanceof String))) {
								items.clear();
								break;
							}
							item.setLtime((String) tmpObj);
							tmpObj = vi.get(1);
							if (tmpObj == null || (!(tmpObj instanceof Double))) {
								items.clear();
								break;
							}
							item.setLvalue(((Double) tmpObj).intValue());
							tmpObj = vs.get(1);
							if (tmpObj == null || (!(tmpObj instanceof List)) || (2 != ((List<?>) tmpObj).size())) {
								items.clear();
								break;
							}
							vi = (List<Object>) tmpObj;
							tmpObj = vi.get(0);
							if (tmpObj == null || (!(tmpObj instanceof String))) {
								items.clear();
								break;
							}
							item.setHtime((String) tmpObj);
							tmpObj = vi.get(1);
							if (tmpObj == null || (!(tmpObj instanceof Double))) {
								items.clear();
								break;
							}
							item.setHvalue(((Double) tmpObj).intValue());
							items.add(item);
						} else {
							items.clear();
							break;
						}
					}
				}
			}
		} finally {
			in.close();
		}
		response.setDateHeader("Expires", 0);
		response.setHeader("Cache-Control", "no-store, no-cache, must-revalidate");
		response.addHeader("Cache-Control", "post-check=0, pre-check=0");
		response.setHeader("Pragma", "no-cache");
		if (null == request.getCharacterEncoding()) {
			request.setCharacterEncoding("UTF-8");
			request.setCharacterEncoding("UTF-8");
		}
		response.setContentType("application/json");
		java.io.PrintWriter out = response.getWriter();
		if (items.isEmpty()) {
			out.write("{\"status\":0,}");
			return;
		}

		List<WaveDataItem> olist = null;
		synchronized (DATA_CACHE) {
			olist = DATA_CACHE.get(serverId);
		}
		if (olist == null || olist.get(0).getCtime().compareTo(ctime) < 0) {
			synchronized (DATA_CACHE) {
				DATA_CACHE.put(serverId, items);
			}
			JfwAppContext.getScheduledExecutorService().execute(new Runnable() {
				@Override
				public void run() {
					try {
						Connection con = dataSource.getConnection();
						try {
							collectDao.insert(con, items);
							con.commit();
						} catch (SQLException e) {
							con.rollback();
						} finally {
							con.close();
						}
					} catch (Throwable thr) {
					}

				}
			});
		}
		out.write("{\"status\":1,}");
		return;
	}

	@Override
	public void init() throws ServletException {
		this.dataSource = JfwAppContext.getDataSource();
		if (this.dataSource == null) {
			throw new ServletException("not found datasource");
		}
		this.collectDao = (CollectDao) JfwAppContext.getBeanFactory().getBean("com_ekexiu_project_bridge_collect_CollectDao");
		if (this.dataSource == null) {
			throw new ServletException("not found collectDao");
		}
		CollectService s = (CollectService) JfwAppContext.getBeanFactory().getBean("com_ekexiu_project_bridge_collect_CollectService");
		if (s == null) {
			throw new ServletException("not found CollectService");
		}
		s.setWaveCache(DATA_CACHE);
	}

	public static void main(String args[]) throws Exception {
		Map<Object, Object> map = JsonService.fromJson("{\r\n" + "\"node_id\": \"12345\",\r\n" + "    \"data\": [	\r\n" + "        {		\r\n"
				+ "            \"box_id\": 1, \r\n" + "            \"channel_id\": 1, 	\r\n" + "            \"value\": [	\r\n" + "                [\r\n"
				+ "                    \"20180101101010.000\",	\r\n" + "                    -12	\r\n" + "                ], \r\n" + "                [\r\n"
				+ "                    \"20180101101011.500\", \r\n" + "                    13			\r\n" + "                ]\r\n" + "            ]\r\n"
				+ "        }, \r\n" + "        {\r\n" + "            \"box_id\": 2, \r\n" + "            \"channel_id\": 9, \r\n"
				+ "            \"value\": [\r\n" + "                [\r\n" + "                    \"20180101101010.100\", \r\n" + "                    -2\r\n"
				+ "                ], \r\n" + "                [\r\n" + "                    \"20180101101011.200\", \r\n" + "                    34\r\n"
				+ "                ]\r\n" + "            ]\r\n" + "        }\r\n" + "    ]\r\n" + "}", DATA_TYPE);

		outMap(0, map);

	}

	@SuppressWarnings("unchecked")
	public static void outList(int left, List<Object> list) {
		for (Object obj : list) {
			for (int i = 0; i < left; ++i) {
				System.out.print(" ");
			}
			System.out.print(obj.getClass().getName() + "  ");
			if (obj instanceof List) {
				System.out.println("===>List");
				outList(left + 8, (List<Object>) obj);
			} else if (obj instanceof Map) {
				System.out.println("===>Map");
				outMap(left + 8, (Map<Object, Object>) obj);
			} else {
				System.out.println("===>" + ((obj instanceof Double) ? ((Double) obj).intValue() : obj.toString()));
			}

		}

	}

	@SuppressWarnings("unchecked")
	public static void outMap(int left, Map<Object, Object> map) {
		for (Map.Entry<Object, Object> entry : map.entrySet()) {
			for (int i = 0; i < left; ++i) {
				System.out.print(" ");
			}
			Object obj = entry.getKey();
			System.out.println("KEY:" + obj.getClass().getName() + "    " + obj.toString());
			for (int i = 0; i < left; ++i) {
				System.out.print(" ");
			}
			obj = entry.getValue();
			System.out.print("VALUE:");
			if (obj instanceof List) {
				System.out.println("===>List");
				outList(left + 8, (List<Object>) obj);
			} else if (obj instanceof Map) {
				System.out.println("===>Map");
				outMap(left + 8, (Map<Object, Object>) obj);
			} else {
				System.out.println("===>" + obj.getClass().getName() + "       " + obj.toString());
			}

		}
	}

}